"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const workspace_1 = require("@teambit/workspace");
const ui_1 = require("@teambit/ui");
const webpack_1 = require("@teambit/webpack");
const bundler_1 = require("@teambit/bundler");
const compiler_1 = require("@teambit/compiler");
const react_1 = __importDefault(require("react"));
const ink_1 = require("ink");
const open_1 = __importDefault(require("open"));
const output_templates_1 = require("./output-templates");
class CliOutput extends react_1.default.Component {
    constructor(props) {
        super(props);
        this.eventsListener = (event) => {
            switch (event.type) {
                case bundler_1.ComponentsServerStartedEvent.TYPE:
                    this.changeOrAddComponentServer(event.data, event.data.context.id, 'Running');
                    if (this.areAllComponentServersRunning()) {
                        this.safeOpenBrowser();
                    }
                    break;
                case webpack_1.WebpackCompilationDoneEvent.TYPE:
                    this.onWebpackCompilationDone(event);
                    break;
                case ui_1.UiServerStartedEvent.TYPE:
                    this.onUiServerStarted(event);
                    break;
                case workspace_1.OnComponentChangeEvent.TYPE:
                    this.onComponentChange(event);
                    break;
                case workspace_1.OnComponentAddEvent.TYPE:
                    this.onComponentAdd(event);
                    break;
                case workspace_1.OnComponentRemovedEvent.TYPE:
                    this.onComponentRemoved(event);
                    break;
                case compiler_1.CompilerErrorEvent.TYPE:
                    this.setState({
                        latestError: event.data.error,
                    });
                    break;
                default:
            }
        };
        this.onWebpackCompilationDone = (event) => {
            this.setState({
                webpackErrors: [...event.data.stats.compilation.errors],
                webpackWarnings: [...event.data.stats.compilation.warnings],
            });
        };
        this.onComponentRemoved = (event) => {
            this.onComponentChange(event);
        };
        this.onComponentAdd = (event) => {
            this.onComponentChange(event);
        };
        this.state = {
            commandFlags: props.commandFlags,
            mainUIServer: null,
            componentServers: [],
            componentChanges: [],
            isBrowserOpen: false,
            latestError: null,
            webpackErrors: [],
            webpackWarnings: [],
        };
        this.registerToEvents(props.pubsub);
    }
    registerToEvents(pubsub) {
        pubsub.sub(ui_1.UIAspect.id, this.eventsListener);
        pubsub.sub(webpack_1.WebpackAspect.id, this.eventsListener);
        pubsub.sub(bundler_1.BundlerAspect.id, this.eventsListener);
        pubsub.sub(workspace_1.WorkspaceAspect.id, this.eventsListener);
        pubsub.sub(compiler_1.CompilerAspect.id, this.eventsListener);
    }
    onUiServerStarted(event) {
        return __awaiter(this, void 0, void 0, function* () {
            const devServers = yield event.data.uiRoot.devServers;
            devServers.forEach((server) => {
                this.changeOrAddComponentServer(server, server.context.id, 'Started');
            });
            this.setState({
                mainUIServer: event.data,
            });
            if (!devServers.length) {
                this.safeOpenBrowser();
            }
        });
    }
    onComponentChange(event) {
        this.setState({
            componentChanges: [...this.state.componentChanges, event],
            latestError: null,
        });
    }
    // Helpers
    areAllComponentServersRunning() {
        return this.state.componentServers.every((cs) => cs.status === 'Running');
    }
    safeOpenBrowser() {
        const { suppressBrowserLaunch } = this.state.commandFlags;
        const { isBrowserOpen, mainUIServer } = this.state;
        if (mainUIServer && !isBrowserOpen && !suppressBrowserLaunch) {
            this.setState({ isBrowserOpen: true });
            setTimeout(() => open_1.default(`http://${mainUIServer.targetHost}:${mainUIServer.targetPort}/`), 500);
        }
    }
    changeOrAddComponentServer(server, id, status) {
        this.setState({
            componentServers: [...this.state.componentServers.filter((cs) => cs.id !== id), { server, id, status }],
        });
    }
    render() {
        const { componentServers, mainUIServer, latestError, webpackErrors, webpackWarnings } = this.state;
        const { verbose } = this.state.commandFlags;
        // run in scope
        if (mainUIServer && mainUIServer.uiRoot.scope) {
            return react_1.default.createElement(output_templates_1.UIServersAreReady, { mainUIServer: mainUIServer });
        }
        return (react_1.default.createElement(react_1.default.Fragment, null,
            mainUIServer ? null : react_1.default.createElement(output_templates_1.Starting, { componentServers: componentServers }),
            react_1.default.createElement(ink_1.Newline, null),
            react_1.default.createElement(output_templates_1.ComponentPreviewServerStarted, { items: componentServers }),
            react_1.default.createElement(ink_1.Newline, null),
            react_1.default.createElement(output_templates_1.TSErrors, { latestError: latestError, verbose: !!verbose }),
            webpackErrors.map((err, index) => (react_1.default.createElement(ink_1.Text, { key: index },
                "Error: ",
                err))),
            webpackWarnings.map((warning, index) => (react_1.default.createElement(ink_1.Text, { key: index },
                "Warning: ",
                warning))),
            react_1.default.createElement(output_templates_1.UIServersAreReady, { mainUIServer: mainUIServer })));
    }
}
exports.CliOutput = CliOutput;
//# sourceMappingURL=cli-output.js.map