"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const workspace_1 = require("@teambit/workspace");
const react_1 = __importDefault(require("react"));
const cli_output_1 = require("./cli-output");
const output_templates_1 = require("./output-templates");
const report_1 = require("./report");
class StartCmd {
    constructor(
    /**
     * access to the extension instance.
     */
    ui, logger, pubsub) {
        this.ui = ui;
        this.logger = logger;
        this.pubsub = pubsub;
        this.name = 'start [type] [pattern]';
        this.description = 'Start a dev environment for a workspace or a specific component';
        this.alias = 'c';
        this.group = 'component';
        this.shortDescription = '';
        this.options = [
            ['d', 'dev', 'start UI server in dev mode.'],
            ['p', 'port', 'port of the UI server.'],
            ['r', 'rebuild', 'rebuild the UI'],
            ['v', 'verbose', 'showing verbose output for inspection and prints stack trace'],
            ['', 'suppress-browser-launch', 'do not automatically open browser when ready'],
        ];
    }
    report([uiRootName, userPattern], { dev, port, rebuild, }) {
        return __awaiter(this, void 0, void 0, function* () {
            return report_1.report([uiRootName, userPattern], { dev, port, rebuild }, this.ui, this.logger, this.pubsub);
        });
    }
    render([uiRootName, userPattern], { dev, port, rebuild, verbose, suppressBrowserLaunch, }) {
        return __awaiter(this, void 0, void 0, function* () {
            this.startingtimestamp = Date.now();
            const pattern = userPattern && userPattern.toString();
            this.logger.off();
            this.ui.createRuntime({
                uiRootName,
                pattern,
                dev,
                port: port ? parseInt(port) : undefined,
                rebuild,
            });
            return (react_1.default.createElement(react_1.default.Fragment, null,
                react_1.default.createElement(output_templates_1.ClearConsole, { verbose: !!verbose }),
                react_1.default.createElement(cli_output_1.CliOutput, { workspaceID: workspace_1.WorkspaceAspect.id, startingTimestamp: Date.now(), pubsub: this.pubsub, commandFlags: { dev: !!dev, port, verbose: !!verbose, suppressBrowserLaunch: !!suppressBrowserLaunch } })));
        });
    }
}
exports.StartCmd = StartCmd;
//# sourceMappingURL=index.js.map